@@ -12,8 +12,10 @@
 
 #include "../company_type.h"
 
+void ShowAIListWindow(CompanyID slot);
 Window* ShowAIDebugWindow(CompanyID show_company = INVALID_COMPANY);
 void ShowAIConfigWindow();
+void ShowScriptTextfileWindow(TextfileType file_type, CompanyID slot);
 void ShowAIDebugWindowIfAIError();
 void InitializeAIGui();
 
