@@ -18,6 +18,7 @@
 #include "ai.hpp"
 
 #include "../script/script_storage.hpp"
+#include "../script/script_cmd.h"
 #include "ai_info.hpp"
 #include "ai_instance.hpp"
 
@@ -60,6 +61,9 @@ void AIInstance::Died()
 {
 	ScriptInstance::Died();
 
+	/* Intro is not supposed to use AI, but it may have 'dummy' AI which instant dies. */
+	if (_game_mode == GM_MENU) return;
+
 	ShowAIDebugWindow(_current_company);
 
 	const AIInfo *info = AIConfig::GetConfig(_current_company, AIConfig::SSS_FORCE_GAME)->GetInfo();
@@ -92,13 +96,13 @@ ScriptInfo *AIInstance::FindLibrary(const char *library, int version)
 
 /**
  * DoCommand callback function for all commands executed by AIs.
+ * @param cmd cmd as given to DoCommandPInternal.
  * @param result The result of the command.
  * @param tile The tile on which the command was executed.
- * @param p1 p1 as given to DoCommandPInternal.
- * @param p2 p2 as given to DoCommandPInternal.
- * @param cmd cmd as given to DoCommandPInternal.
+ * @param data Command data as given to Command<>::Post.
+ * @param result_data Additional returned data from the command.
  */
-void CcAI(const CommandCost &result, TileIndex tile, uint32 p1, uint32 p2, uint32 cmd)
+void CcAI(Commands cmd, const CommandCost &result, TileIndex tile, const CommandDataBuffer &data, CommandDataBuffer result_data)
 {
 	/*
 	 * The company might not exist anymore. Check for this.
@@ -109,12 +113,12 @@ void CcAI(const CommandCost &result, TileIndex tile, uint32 p1, uint32 p2, uint3
 	const Company *c = Company::GetIfValid(_current_company);
 	if (c == nullptr || c->ai_instance == nullptr) return;
 
-	if (c->ai_instance->DoCommandCallback(result, tile, p1, p2, cmd)) {
+	if (c->ai_instance->DoCommandCallback(result, tile, data, std::move(result_data), cmd)) {
 		c->ai_instance->Continue();
 	}
 }
 
-CommandCallback *AIInstance::GetDoCommandCallback()
+CommandCallbackData *AIInstance::GetDoCommandCallback()
 {
 	return &CcAI;
 }
