@@ -234,7 +234,7 @@ public:
 						if (x * new_height > new_capacity) continue;
 						(*copy)(new_data + (x - 1) * new_height,
 								this->data + (x - 1) * this->height,
-								min(this->height, new_height));
+								std::min(this->height, new_height));
 					}
 				} else {
 					/* If matrix is shrinking copy from the front. */
@@ -242,7 +242,7 @@ public:
 						if ((x + 1) * new_height > new_capacity) break;
 						(*copy)(new_data + x * new_height,
 								this->data + x * this->height,
-								min(this->height, new_height));
+								std::min(this->height, new_height));
 					}
 				}
 			}
