@@ -29,12 +29,12 @@ protected:
 	int descender;                    ///< The descender value of the font.
 	int units_per_em;                 ///< The units per EM value of the font.
 
-	static int GetDefaultFontHeight(FontSize fs);
-
 public:
 	FontCache(FontSize fs);
 	virtual ~FontCache();
 
+	static int GetDefaultFontHeight(FontSize fs);
+
 	/**
 	 * Get the FontSize of the font.
 	 * @return The FontSize.
@@ -71,13 +71,6 @@ public:
 	 */
 	virtual int GetFontSize() const { return this->height; }
 
-	/**
-	 * Get the SpriteID mapped to the given key
-	 * @param key The key to get the sprite for.
-	 * @return The sprite.
-	 */
-	virtual SpriteID GetUnicodeGlyph(WChar key) = 0;
-
 	/**
 	 * Map a SpriteID to the key
 	 * @param key The key to map to.
@@ -166,12 +159,6 @@ public:
 	virtual bool IsBuiltInFont() = 0;
 };
 
-/** Get the SpriteID mapped to the given font size and key */
-static inline SpriteID GetUnicodeGlyph(FontSize size, WChar key)
-{
-	return FontCache::Get(size)->GetUnicodeGlyph(key);
-}
-
 /** Map a SpriteID to the font size and key */
 static inline void SetUnicodeGlyph(FontSize size, WChar key, SpriteID sprite)
 {
@@ -212,8 +199,8 @@ static inline bool GetDrawGlyphShadow(FontSize size)
 	return FontCache::Get(size)->GetDrawGlyphShadow();
 }
 
-/** Settings for a single freetype font. */
-struct FreeTypeSubSetting {
+/** Settings for a single font. */
+struct FontCacheSubSetting {
 	std::string font; ///< The name of the font, or path to the font.
 	uint size;        ///< The (requested) size of the font.
 	bool aa;          ///< Whether to do anti aliasing or not.
@@ -221,18 +208,37 @@ struct FreeTypeSubSetting {
 	const void *os_handle = nullptr; ///< Optional native OS font info. Only valid during font search.
 };
 
-/** Settings for the freetype fonts. */
-struct FreeTypeSettings {
-	FreeTypeSubSetting small;  ///< The smallest font; mostly used for zoomed out view.
-	FreeTypeSubSetting medium; ///< The normal font size.
-	FreeTypeSubSetting large;  ///< The largest font; mostly used for newspapers.
-	FreeTypeSubSetting mono;   ///< The mono space font used for license/readme viewers.
+/** Settings for the four different fonts. */
+struct FontCacheSettings {
+	FontCacheSubSetting small;  ///< The smallest font; mostly used for zoomed out view.
+	FontCacheSubSetting medium; ///< The normal font size.
+	FontCacheSubSetting large;  ///< The largest font; mostly used for newspapers.
+	FontCacheSubSetting mono;   ///< The mono space font used for license/readme viewers.
 };
 
-extern FreeTypeSettings _freetype;
+extern FontCacheSettings _fcsettings;
 
-void InitFreeType(bool monospace);
-void UninitFreeType();
+/**
+ * Get the settings of a given font size.
+ * @param fs The font size to look up.
+ * @return The settings.
+ */
+static inline FontCacheSubSetting *GetFontCacheSubSetting(FontSize fs)
+{
+	switch (fs) {
+		default: NOT_REACHED();
+		case FS_SMALL:  return &_fcsettings.small;
+		case FS_NORMAL: return &_fcsettings.medium;
+		case FS_LARGE:  return &_fcsettings.large;
+		case FS_MONO:   return &_fcsettings.mono;
+	}
+}
+
+void InitFontCache(bool monospace);
+void UninitFontCache();
 bool HasAntialiasedFonts();
 
+bool GetFontAAState(FontSize size, bool check_blitter = true);
+void SetFont(FontSize fontsize, const std::string &font, uint size, bool aa);
+
 #endif /* FONTCACHE_H */
