@@ -13,6 +13,7 @@
 #include "../script/squirrel_class.hpp"
 
 #include "../script/script_storage.hpp"
+#include "../script/script_cmd.h"
 #include "../ai/ai_gui.hpp"
 #include "game_config.hpp"
 #include "game_info.hpp"
@@ -81,20 +82,20 @@ void GameInstance::Died()
 
 /**
  * DoCommand callback function for all commands executed by Game Scripts.
+ * @param cmd cmd as given to DoCommandPInternal.
  * @param result The result of the command.
  * @param tile The tile on which the command was executed.
- * @param p1 p1 as given to DoCommandPInternal.
- * @param p2 p2 as given to DoCommandPInternal.
- * @param cmd cmd as given to DoCommandPInternal.
+ * @param data Command data as given to Command<>::Post.
+ * @param result_data Additional returned data from the command.
  */
-void CcGame(const CommandCost &result, TileIndex tile, uint32 p1, uint32 p2, uint32 cmd)
+void CcGame(Commands cmd, const CommandCost &result, TileIndex tile, const CommandDataBuffer &data, CommandDataBuffer result_data)
 {
-	if (Game::GetGameInstance()->DoCommandCallback(result, tile, p1, p2, cmd)) {
+	if (Game::GetGameInstance()->DoCommandCallback(result, tile, data, std::move(result_data), cmd)) {
 		Game::GetGameInstance()->Continue();
 	}
 }
 
-CommandCallback *GameInstance::GetDoCommandCallback()
+CommandCallbackData *GameInstance::GetDoCommandCallback()
 {
 	return &CcGame;
 }
