@@ -45,7 +45,7 @@ static const char * GetGamelogRevisionString()
 {
 	/* Allocate a buffer larger than necessary (git revision hash is 40 bytes) to avoid truncation later */
 	static char gamelog_revision[48] = { 0 };
-	assert_compile(lengthof(gamelog_revision) > GAMELOG_REVISION_LENGTH);
+	static_assert(lengthof(gamelog_revision) > GAMELOG_REVISION_LENGTH);
 
 	if (IsReleasedVersion()) {
 		return _openttd_revision;
@@ -167,7 +167,7 @@ static const char * const la_text[] = {
 	"emergency savegame",
 };
 
-assert_compile(lengthof(la_text) == GLAT_END);
+static_assert(lengthof(la_text) == GLAT_END);
 
 /**
  * Information about the presence of a Grf at a certain point during gamelog history
@@ -356,7 +356,7 @@ static int _gamelog_print_level = 0; ///< gamelog debug level we need to print s
 
 static void GamelogPrintDebugProc(const char *s)
 {
-	DEBUG(gamelog, _gamelog_print_level, "%s", s);
+	Debug(gamelog, _gamelog_print_level, "{}", s);
 }
 
 
@@ -483,14 +483,14 @@ void GamelogOldver()
  * @param oldval old setting value
  * @param newval new setting value
  */
-void GamelogSetting(const char *name, int32 oldval, int32 newval)
+void GamelogSetting(const std::string &name, int32 oldval, int32 newval)
 {
 	assert(_gamelog_action_type == GLAT_SETTING);
 
 	LoggedChange *lc = GamelogChange(GLCT_SETTING);
 	if (lc == nullptr) return;
 
-	lc->setting.name = stredup(name);
+	lc->setting.name = stredup(name.c_str());
 	lc->setting.oldval = oldval;
 	lc->setting.newval = newval;
 }
@@ -814,7 +814,7 @@ void GamelogInfo(LoggedAction *gamelog_action, uint gamelog_actions, uint32 *las
 
 				case GLCT_REVISION:
 					*last_ottd_rev = lc->revision.newgrf;
-					*ever_modified = max(*ever_modified, lc->revision.modified);
+					*ever_modified = std::max(*ever_modified, lc->revision.modified);
 					break;
 
 				case GLCT_GRFREM:
