@@ -20,7 +20,6 @@
 struct Window;
 
 /* main_gui.cpp */
-void HandleOnEditText(const char *str);
 void InitializeGUI();
 
 /* settings_gui.cpp */
@@ -55,8 +54,8 @@ void ShowStoryBook(CompanyID company, uint16 page_id = INVALID_STORY_PAGE);
 
 void ShowEstimatedCostOrIncome(Money cost, int x, int y);
 
-void ShowExtraViewPortWindow(TileIndex tile = INVALID_TILE);
-void ShowExtraViewPortWindowForTileUnderCursor();
+void ShowExtraViewportWindow(TileIndex tile = INVALID_TILE);
+void ShowExtraViewportWindowForTileUnderCursor();
 
 /* bridge_gui.cpp */
 void ShowBuildBridgeWindow(TileIndex start, TileIndex end, TransportType transport_type, byte bridge_type);
