@@ -39,7 +39,7 @@ private:
 	~LinkGraphSchedule();
 	typedef std::list<LinkGraph *> GraphList;
 	typedef std::list<LinkGraphJob *> JobList;
-	friend const SaveLoad *GetLinkGraphScheduleDesc();
+	friend SaveLoadTable GetLinkGraphScheduleDesc();
 
 protected:
 	ComponentHandler *handlers[6]; ///< Handlers to be run for each job.
@@ -55,6 +55,7 @@ public:
 	static void Clear();
 
 	void SpawnNext();
+	bool IsJoinWithUnfinishedJobDue() const;
 	void JoinNext();
 	void SpawnAll();
 	void ShiftDates(int interval);
@@ -76,4 +77,7 @@ public:
 	void Unqueue(LinkGraph *lg) { this->schedule.remove(lg); }
 };
 
+void StateGameLoop_LinkGraphPauseControl();
+void AfterLoad_LinkGraphPauseControl();
+
 #endif /* LINKGRAPHSCHEDULE_H */
