@@ -58,40 +58,16 @@ static void DoSetVolume()
 		AUGraphGetIndNode(graph, i, &node);
 
 		AudioUnit unit;
-		OSType comp_type = 0;
+		AudioComponentDescription desc;
+		AUGraphNodeInfo(graph, node, &desc, &unit);
 
-#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5)
-		if (MacOSVersionIsAtLeast(10, 5, 0)) {
-			/* The 10.6 SDK has changed the function prototype of
-			 * AUGraphNodeInfo. This is a binary compatible change,
-			 * but we need to get the type declaration right or
-			 * risk compilation errors. The header AudioComponent.h
-			 * was introduced in 10.6 so use it to decide which
-			 * type definition to use. */
-#if defined(__AUDIOCOMPONENT_H__) || defined(HAVE_OSX_107_SDK)
-			AudioComponentDescription desc;
-#else
-			ComponentDescription desc;
-#endif
-			AUGraphNodeInfo(graph, node, &desc, &unit);
-			comp_type = desc.componentType;
-		} else
-#endif
-		{
-#if (MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_5)
-			ComponentDescription desc;
-			AUGraphGetNodeInfo(graph, node, &desc, nullptr, nullptr, &unit);
-			comp_type = desc.componentType;
-#endif
-		}
-
-		if (comp_type == kAudioUnitType_Output) {
+		if (desc.componentType == kAudioUnitType_Output) {
 			output_unit = unit;
 			break;
 		}
 	}
 	if (output_unit == nullptr) {
-		DEBUG(driver, 1, "cocoa_m: Failed to get output node to set volume");
+		Debug(driver, 1, "cocoa_m: Failed to get output node to set volume");
 		return;
 	}
 
@@ -103,7 +79,7 @@ static void DoSetVolume()
 /**
  * Initialized the MIDI player, including QuickTime initialization.
  */
-const char *MusicDriver_Cocoa::Start(const char * const *parm)
+const char *MusicDriver_Cocoa::Start(const StringList &parm)
 {
 	if (NewMusicPlayer(&_player) != noErr) return "failed to create music player";
 
@@ -143,7 +119,7 @@ void MusicDriver_Cocoa::PlaySong(const MusicSongInfo &song)
 {
 	std::string filename = MidiFile::GetSMFFile(song);
 
-	DEBUG(driver, 2, "cocoa_m: trying to play '%s'", filename.c_str());
+	Debug(driver, 2, "cocoa_m: trying to play '{}'", filename);
 
 	this->StopSong();
 	if (_sequence != nullptr) {
@@ -154,33 +130,16 @@ void MusicDriver_Cocoa::PlaySong(const MusicSongInfo &song)
 	if (filename.empty()) return;
 
 	if (NewMusicSequence(&_sequence) != noErr) {
-		DEBUG(driver, 0, "cocoa_m: Failed to create music sequence");
+		Debug(driver, 0, "cocoa_m: Failed to create music sequence");
 		return;
 	}
 
-	const char *os_file = OTTD2FS(filename.c_str());
-	CFAutoRelease<CFURLRef> url(CFURLCreateFromFileSystemRepresentation(kCFAllocatorDefault, (const UInt8*)os_file, strlen(os_file), false));
+	std::string os_file = OTTD2FS(filename);
+	CFAutoRelease<CFURLRef> url(CFURLCreateFromFileSystemRepresentation(kCFAllocatorDefault, (const UInt8*)os_file.c_str(), os_file.length(), false));
 
-#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5)
-	if (MacOSVersionIsAtLeast(10, 5, 0)) {
-		if (MusicSequenceFileLoad(_sequence, url.get(), kMusicSequenceFile_AnyType, 0) != noErr) {
-			DEBUG(driver, 0, "cocoa_m: Failed to load MIDI file");
-			return;
-		}
-	} else
-#endif
-	{
-#if (MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_5)
-		FSRef ref_file;
-		if (!CFURLGetFSRef(url.get(), &ref_file)) {
-			DEBUG(driver, 0, "cocoa_m: Failed to make FSRef");
-			return;
-		}
-		if (MusicSequenceLoadSMFWithFlags(_sequence, &ref_file, 0) != noErr) {
-			DEBUG(driver, 0, "cocoa_m: Failed to load MIDI file old style");
-			return;
-		}
-#endif
+	if (MusicSequenceFileLoad(_sequence, url.get(), kMusicSequenceFile_AnyType, 0) != noErr) {
+		Debug(driver, 0, "cocoa_m: Failed to load MIDI file");
+		return;
 	}
 
 	/* Construct audio graph */
@@ -189,7 +148,7 @@ void MusicDriver_Cocoa::PlaySong(const MusicSongInfo &song)
 	MusicSequenceGetAUGraph(_sequence, &graph);
 	AUGraphOpen(graph);
 	if (AUGraphInitialize(graph) != noErr) {
-		DEBUG(driver, 0, "cocoa_m: Failed to initialize AU graph");
+		Debug(driver, 0, "cocoa_m: Failed to initialize AU graph");
 		return;
 	}
 
@@ -214,7 +173,7 @@ void MusicDriver_Cocoa::PlaySong(const MusicSongInfo &song)
 	if (MusicPlayerStart(_player) != noErr) return;
 	_playing = true;
 
-	DEBUG(driver, 3, "cocoa_m: playing '%s'", filename.c_str());
+	Debug(driver, 3, "cocoa_m: playing '{}'", filename);
 }
 
 
