@@ -14,7 +14,7 @@
 
 class MusicDriver_Cocoa : public MusicDriver {
 public:
-	const char *Start(const char * const *param) override;
+	const char *Start(const StringList &param) override;
 
 	void Stop() override;
 
