@@ -34,7 +34,7 @@ bool LoadNewGRFSound(SoundEntry *sound);
 SoundID GetNewGRFSoundID(const struct GRFFile *file, SoundID sound_id);
 SoundEntry *GetSound(SoundID sound_id);
 uint GetNumSounds();
-bool PlayVehicleSound(const Vehicle *v, VehicleSoundEvent event);
+bool PlayVehicleSound(const Vehicle *v, VehicleSoundEvent event, bool force  = false);
 void PlayTileSound(const struct GRFFile *file, SoundID sound_id, TileIndex tile);
 
 #endif /* NEWGRF_SOUND_H */
