@@ -10,10 +10,6 @@
 #ifndef PROGRESS_H
 #define PROGRESS_H
 
-#include <mutex>
-
-static const uint MODAL_PROGRESS_REDRAW_TIMEOUT = 200; ///< Timeout between redraws
-
 /**
  * Check if we are currently in a modal progress state.
  * @return Are we in the modal state?
@@ -24,20 +20,6 @@ static inline bool HasModalProgress()
 	return _in_modal_progress;
 }
 
-/**
- * Check if we can use a thread for modal progress.
- * @return Threading usable?
- */
-static inline bool UseThreadedModelProgress()
-{
-	extern bool _use_threaded_modal_progress;
-	return _use_threaded_modal_progress;
-}
-
-bool IsFirstModalProgressLoop();
 void SetModalProgress(bool state);
 
-extern std::mutex _modal_progress_work_mutex;
-extern std::mutex _modal_progress_paint_mutex;
-
 #endif /* PROGRESS_H */
