@@ -15,10 +15,11 @@
 /** Implementation of the null sound driver. */
 class SoundDriver_Null : public SoundDriver {
 public:
-	const char *Start(const char * const *param) override { return nullptr; }
+	const char *Start(const StringList &param) override { return nullptr; }
 
 	void Stop() override { }
 	const char *GetName() const override { return "null"; }
+	bool HasOutput() const override { return false; }
 };
 
 /** Factory for the null sound driver. */
