@@ -41,9 +41,10 @@ enum StationType {
 };
 
 /** Types of RoadStops */
-enum RoadStopType {
+enum RoadStopType : byte {
 	ROADSTOP_BUS,    ///< A standard stop for buses
 	ROADSTOP_TRUCK,  ///< A standard stop for trucks
+	ROADSTOP_END,    ///< End of valid types
 };
 
 /** The facilities a station might be having */
