@@ -19,6 +19,7 @@
 #include <stdarg.h>
 #include <ctype.h> /* required for tolower() */
 #include <sstream>
+#include <iomanip>
 
 #ifdef _MSC_VER
 #include <errno.h> // required by vsnprintf implementation for MSVC
@@ -160,6 +161,23 @@ char *CDECL str_fmt(const char *str, ...)
 	return p;
 }
 
+/**
+ * Format a byte array into a continuous hex string.
+ * @param data Array to format
+ * @return Converted string.
+ */
+std::string FormatArrayAsHex(span<const byte> data)
+{
+	std::ostringstream ss;
+	ss << std::uppercase << std::setfill('0') << std::setw(2) << std::hex;
+
+	for (auto b : data) {
+		ss << b;
+	}
+
+	return ss.str();
+}
+
 /**
  * Scan the string for old values of SCC_ENCODED and fix it to
  * it's new, static value.
@@ -476,11 +494,12 @@ bool strtolower(std::string &str, std::string::size_type offs)
 bool IsValidChar(WChar key, CharSetFilter afilter)
 {
 	switch (afilter) {
-		case CS_ALPHANUMERAL:  return IsPrintable(key);
-		case CS_NUMERAL:       return (key >= '0' && key <= '9');
-		case CS_NUMERAL_SPACE: return (key >= '0' && key <= '9') || key == ' ';
-		case CS_ALPHA:         return IsPrintable(key) && !(key >= '0' && key <= '9');
-		case CS_HEXADECIMAL:   return (key >= '0' && key <= '9') || (key >= 'a' && key <= 'f') || (key >= 'A' && key <= 'F');
+		case CS_ALPHANUMERAL:   return IsPrintable(key);
+		case CS_NUMERAL:        return (key >= '0' && key <= '9');
+		case CS_NUMERAL_SPACE:  return (key >= '0' && key <= '9') || key == ' ';
+		case CS_NUMERAL_SIGNED: return (key >= '0' && key <= '9') || key == '-';
+		case CS_ALPHA:          return IsPrintable(key) && !(key >= '0' && key <= '9');
+		case CS_HEXADECIMAL:    return (key >= '0' && key <= '9') || (key >= 'a' && key <= 'f') || (key >= 'A' && key <= 'F');
 		default: NOT_REACHED();
 	}
 }
