@@ -406,7 +406,7 @@ extern const AirportSpec _origin_airport_specs[] = {
 	AS_GENERIC(&_airportfta_oilrig, nullptr, _default_airports_rotation, 0, nullptr, 0, 1, 1, 0, 4, 0, 0, 0, ATP_TTDP_OILRIG, APC_HELIPORT, STR_NULL, 0, false),
 };
 
-assert_compile(NEW_AIRPORT_OFFSET == lengthof(_origin_airport_specs));
+static_assert(NEW_AIRPORT_OFFSET == lengthof(_origin_airport_specs));
 
 const AirportSpec AirportSpec::dummy = AS_GENERIC(&_airportfta_dummy, nullptr, _default_airports_rotation, 0, nullptr, 0, 0, 0, 0, 0, MIN_YEAR, MIN_YEAR, 0, ATP_TTDP_LARGE, APC_BEGIN, STR_NULL, 0, false);
 
