@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -12,6 +10,7 @@
 #ifndef TUNNEL_MAP_H
 #define TUNNEL_MAP_H
 
+#include "rail_map.h"
 #include "road_map.h"
 
 
@@ -48,7 +47,7 @@ bool IsTunnelInWayDir(TileIndex tile, int z, DiagDirection dir);
  * @param d the direction facing out of the tunnel
  * @param r the road type used in the tunnel
  */
-static inline void MakeRoadTunnel(TileIndex t, Owner o, DiagDirection d, RoadTypes r)
+static inline void MakeRoadTunnel(TileIndex t, Owner o, DiagDirection d, RoadType road_rt, RoadType tram_rt)
 {
 	SetTileType(t, MP_TUNNELBRIDGE);
 	SetTileOwner(t, o);
@@ -59,9 +58,9 @@ static inline void MakeRoadTunnel(TileIndex t, Owner o, DiagDirection d, RoadTyp
 	SB(_me[t].m6, 2, 4, 0);
 	_me[t].m7 = 0;
 	_me[t].m8 = 0;
-	SetRoadOwner(t, ROADTYPE_ROAD, o);
-	if (o != OWNER_TOWN) SetRoadOwner(t, ROADTYPE_TRAM, o);
-	SetRoadTypes(t, r);
+	SetRoadOwner(t, RTT_ROAD, o);
+	if (o != OWNER_TOWN) SetRoadOwner(t, RTT_TRAM, o);
+	SetRoadTypes(t, road_rt, tram_rt);
 }
 
 /**
@@ -81,7 +80,9 @@ static inline void MakeRailTunnel(TileIndex t, Owner o, DiagDirection d, RailTyp
 	_m[t].m5 = TRANSPORT_RAIL << 2 | d;
 	SB(_me[t].m6, 2, 4, 0);
 	_me[t].m7 = 0;
-	_me[t].m8 = r;
+	_me[t].m8 = 0;
+	SetRailType(t, r);
+	SetRoadTypes(t, INVALID_ROADTYPE, INVALID_ROADTYPE);
 }
 
 #endif /* TUNNEL_MAP_H */
