@@ -115,59 +115,55 @@ const struct Livery *GetEngineLivery(EngineID engine_type, CompanyID company, En
 SpriteID GetEnginePalette(EngineID engine_type, CompanyID company);
 SpriteID GetVehiclePalette(const Vehicle *v);
 
-extern const uint32 _veh_build_proc_table[];
-extern const uint32 _veh_sell_proc_table[];
-extern const uint32 _veh_refit_proc_table[];
-extern const uint32 _send_to_depot_proc_table[];
+extern const StringID _veh_build_msg_table[];
+extern const StringID _veh_sell_msg_table[];
+extern const StringID _veh_refit_msg_table[];
+extern const StringID _send_to_depot_msg_table[];
 
 /* Functions to find the right command for certain vehicle type */
-static inline uint32 GetCmdBuildVeh(VehicleType type)
+static inline StringID GetCmdBuildVehMsg(VehicleType type)
 {
-	return _veh_build_proc_table[type];
+	return _veh_build_msg_table[type];
 }
 
-static inline uint32 GetCmdBuildVeh(const BaseVehicle *v)
+static inline StringID GetCmdBuildVehMsg(const BaseVehicle *v)
 {
-	return GetCmdBuildVeh(v->type);
+	return GetCmdBuildVehMsg(v->type);
 }
 
-static inline uint32 GetCmdSellVeh(VehicleType type)
+static inline StringID GetCmdSellVehMsg(VehicleType type)
 {
-	return _veh_sell_proc_table[type];
+	return _veh_sell_msg_table[type];
 }
 
-static inline uint32 GetCmdSellVeh(const BaseVehicle *v)
+static inline StringID GetCmdSellVehMsg(const BaseVehicle *v)
 {
-	return GetCmdSellVeh(v->type);
+	return GetCmdSellVehMsg(v->type);
 }
 
-static inline uint32 GetCmdRefitVeh(VehicleType type)
+static inline StringID GetCmdRefitVehMsg(VehicleType type)
 {
-	return _veh_refit_proc_table[type];
+	return _veh_refit_msg_table[type];
 }
 
-static inline uint32 GetCmdRefitVeh(const BaseVehicle *v)
+static inline StringID GetCmdRefitVehMsg(const BaseVehicle *v)
 {
-	return GetCmdRefitVeh(v->type);
+	return GetCmdRefitVehMsg(v->type);
 }
 
-static inline uint32 GetCmdSendToDepot(VehicleType type)
+static inline StringID GetCmdSendToDepotMsg(VehicleType type)
 {
-	return _send_to_depot_proc_table[type];
+	return _send_to_depot_msg_table[type];
 }
 
-static inline uint32 GetCmdSendToDepot(const BaseVehicle *v)
+static inline StringID GetCmdSendToDepotMsg(const BaseVehicle *v)
 {
-	return GetCmdSendToDepot(v->type);
+	return GetCmdSendToDepotMsg(v->type);
 }
 
 CommandCost EnsureNoVehicleOnGround(TileIndex tile);
 CommandCost EnsureNoTrainOnTrackBits(TileIndex tile, TrackBits track_bits);
 
-extern VehicleID _new_vehicle_id;
-extern uint _returned_refit_capacity;
-extern uint16 _returned_mail_refit_capacity;
-
 bool CanVehicleUseStation(EngineID engine_type, const struct Station *st);
 bool CanVehicleUseStation(const Vehicle *v, const struct Station *st);
 
@@ -178,4 +174,7 @@ void GetVehicleSet(VehicleSet &set, Vehicle *v, uint8 num_vehicles);
 
 void CheckCargoCapacity(Vehicle *v);
 
+bool VehiclesHaveSameEngineList(const Vehicle *v1, const Vehicle *v2);
+bool VehiclesHaveSameOrderList(const Vehicle *v1, const Vehicle *v2);
+
 #endif /* VEHICLE_FUNC_H */
