@@ -540,6 +540,11 @@ const char *VideoDriver_SDL_Base::Start(const StringList &param)
 	const char *dname = SDL_GetCurrentVideoDriver();
 	Debug(driver, 1, "SDL2: using driver '{}'", dname);
 
+	this->driver_info = this->GetName();
+	this->driver_info += " (";
+	this->driver_info += dname;
+	this->driver_info += ")";
+
 	MarkWholeScreenDirty();
 
 	SDL_StopTextInput();
@@ -643,12 +648,11 @@ bool VideoDriver_SDL_Base::ChangeResolution(int w, int h)
 
 bool VideoDriver_SDL_Base::ToggleFullscreen(bool fullscreen)
 {
-	int w, h;
-
 	/* Remember current window size */
-	if (fullscreen) {
-		SDL_GetWindowSize(this->sdl_window, &w, &h);
+	int w, h;
+	SDL_GetWindowSize(this->sdl_window, &w, &h);
 
+	if (fullscreen) {
 		/* Find fullscreen window size */
 		SDL_DisplayMode dm;
 		if (SDL_GetCurrentDisplayMode(0, &dm) < 0) {
